import unittest
import mock
import sys
import os
import gocept.support.supportmail
import gocept.support.notification
import gocept.support.rt
import gocept.support.utils
import pytz
import StringIO
import datetime


class SupportMailTest(unittest.TestCase):

    @mock.patch('gocept.support.rt.RT.new_ticket')
    @mock.patch('gocept.support.notification.send_email')
    def test_recipient_without_pin_creates_response_but_no_ticket(self, send_email, new_ticket):
        config = gocept.support.configuration.Configuration()
        gocept.support.supportmail.incoming_support_mail(config,
                'standby@gocept.com', {'From': 'dh@gocept.com',
                    'Subject': 'Hilfe',
                    'Message-ID': '12345'})
        sent_mail = send_email.call_args[0][1]
        self.assertEqual('dh@gocept.com', sent_mail['To'])
        self.assertEqual('Re: Hilfe', sent_mail['Subject'])
        self.assertEqual('12345', sent_mail['In-Reply-To'])
        self.assertEqual(1, send_email.call_count)
        self.assertFalse(new_ticket.called)

    @mock.patch('gocept.support.rt.RT.new_ticket')
    @mock.patch('gocept.support.notification.send_email')
    def test_recipient_wrong_pin_creates_response_but_no_ticket(self, send_email, new_ticket):
        config = gocept.support.configuration.Configuration()
        config.add_section('slas')
        gocept.support.supportmail.incoming_support_mail(config,
                'standby+12345@gocept.com', {'From': 'dh@gocept.com',
                    'Subject': 'Hilfe',
                    'Message-ID': '12345'})
        sent_mail = send_email.call_args[0][1]
        self.assertEqual('dh@gocept.com', sent_mail['To'])
        self.assertEqual('Re: Hilfe', sent_mail['Subject'])
        self.assertEqual('12345', sent_mail['In-Reply-To'])
        self.assertEqual(1, send_email.call_count)
        self.assertFalse(new_ticket.called)

    @mock.patch('gocept.support.utils.now')
    @mock.patch('gocept.support.rt.RT.new_ticket')
    @mock.patch('gocept.support.rt.RT.open')
    @mock.patch('gocept.support.notification.send_email')
    @mock.patch('gocept.support.notification.Notifications.notify')
    def test_recipient_currently_no_sla_creates_response_and_ticket_but_no_notification(
            self, notify, send_email, rt_open, new_ticket, now):
        new_ticket.return_value = ticket = mock.Mock()
        ticket.__getitem__ = mock.Mock()
        ticket.__getitem__.return_value = '123'
        rt_open.return_value = StringIO.StringIO()
        now.return_value = datetime.datetime(
            2010,11,8,14,17, tzinfo=pytz.timezone('Europe/Berlin'))
        config = gocept.support.configuration.Configuration()
        config.add_section('rt')
        config.set('rt', 'user', 'foo')
        config.set('rt', 'queue', 'foo')
        config.set('rt', 'password', 'foo')
        config.set('rt', 'url', 'foo')
        config.add_section('slas')
        config.set('slas', 'examplecorp', os.path.join(
            os.path.dirname(__file__), 'data', 'examplecorp.cfg'))
        gocept.support.supportmail.incoming_support_mail(config,
                'standby+12345@gocept.com', {'From': 'dh@gocept.com',
                    'Subject': 'Hilfe',
                    'Message-ID': '12345'})
        self.assertEqual(2, send_email.call_count)
        firstmail, secondmail = send_email.call_args_list
        sent_mail_to_rt = secondmail[0][1]
        sent_mail_to_requestor = firstmail[0][1]
        to_requestor = firstmail[0][0]
        to_rt = secondmail[0][0]
        self.assertEqual('support-comment@gocept.com', to_rt)
        self.assertEqual('dh@gocept.com', sent_mail_to_rt['From'])
        self.assertEqual('[gocept #123] Fw: Hilfe', sent_mail_to_rt['Subject'])
        self.assertEqual('dh@gocept.com', to_requestor)
        self.assertEqual('Re: Hilfe', sent_mail_to_requestor['Subject'])
        self.assertEqual('12345', sent_mail_to_requestor['In-Reply-To'])
        self.assertFalse(ticket.edit.called)
        self.assertTrue(new_ticket.called)
        self.assertFalse(notify.called)

    @mock.patch('gocept.support.utils.now')
    @mock.patch('gocept.support.rt.RT.new_ticket')
    @mock.patch('gocept.support.rt.RT.open')
    @mock.patch('gocept.support.notification.Notifications._load_contacts')
    @mock.patch('gocept.support.notification.send_email')
    @mock.patch('gocept.support.notification.Notifications.notify')
    def test_recipient_with_sla_creates_no_response_but_ticket_and_notification(
            self, notify, send_email, load_contacts, rt_open, new_ticket, now):
        new_ticket.return_value = ticket = mock.Mock()
        ticket.__getitem__ = mock.Mock()
        ticket.__getitem__.return_value = '123'
        rt_open.return_value = StringIO.StringIO()
        now.return_value = datetime.datetime(
            2010,11,8,11,17, tzinfo=pytz.timezone('Europe/Berlin'))
        config = gocept.support.configuration.Configuration()
        config.add_section('rt')
        config.set('rt', 'user', 'foo')
        config.set('rt', 'queue', 'foo')
        config.set('rt', 'password', 'foo')
        config.set('rt', 'url', 'foo')
        config.add_section('main')
        config.add_section('slas')
        config.set('slas', 'examplecorp', os.path.join(
            os.path.dirname(__file__), 'data', 'examplecorp.cfg'))
        gocept.support.supportmail.incoming_support_mail(config,
                'standby+12345@gocept.com', {'From': 'dh@gocept.com',
                    'Subject': 'Hilfe',
                    'Message-ID': '12345'})
        self.assertEqual(1, send_email.call_count)
        firstmail = send_email.call_args
        sent_mail_to_rt = firstmail[0][1]
        to_rt = firstmail[0][0]
        self.assertEqual('support-comment@gocept.com', to_rt)
        self.assertEqual('dh@gocept.com', sent_mail_to_rt['From'])
        self.assertEqual('[gocept #123] Fw: Hilfe', sent_mail_to_rt['Subject'])
        self.assertEqual({'Starts': 'Mon Nov 08 12:17:00 2010'}, ticket.edit.call_args[0][0])
        self.assertTrue(ticket.edit.called)
        self.assertTrue(new_ticket.called)
        self.assertTrue(notify.called)
